#!/bin/sh

################
#auteur: yarel.contrib@tiscali.fr
#	recherche des fichiers doublons à partir du dossier passé en argument
#	parcour les sous-repertoires
#	pour chaque fichier (ne suis pas les liens)
#	recherche des fichiers du même nom dans tous les autres répertoires connu par "locate"
#	puis compare la somme MD5 des fichiers trouvés à celui du répertoire courant
#	déterminant ainsi les "vrai" doublons des "homonymes"
#	ecrit le résultat dans un fichier texte
#NOTE
#	Il est utile de lancer la commande (sous root) "updatedb" avant de lancer
#	le script la première fois ce qui permet de mettre à jour la bd utilisée par "locate"
#
#	si vous exécutez le script plusieurs fois, le résultat est rajouté au fichier
################

#ici mettez le chemin jusqu'au fichier où le résultat sera écrit (il sera créer à la première exécution)
#par défault: "/home/vous/doublons.txt"
LOG=$HOME"/doublons.txt"
NBRREP=0
NBRFICHIER=0
NBRDOUBLON=0
NBRREPDOUB=0

#liste tous les fichiers réguliers du rep
function liste_fichier
{
declare -a tabr


	for fichier in * ;
	do
		#filtre
 		if [ -f "$fichier" ] ; then
 			#sortie console
 			echo  "		fichier -" $fichier

 			#compteur de fichier
 			NBRFICHIER=$((NBRFICHIER + 1))

 			#recherche les doublons (ligne(=chemin complet) finissant par "/nomdufichier")
 			#remplace(temporairement=pour remplir le tableau) les espaces par des ";"
 			tabr=(`locate -r "/$fichier$" | tr ' ' ';'`)

 			#le ndr d'élément
 			nbr_item=${#tabr[@]}

 			#mise en forme si + d'une ligne renvoyée par "slocate..."
 			if [ "$nbr_item" -gt 1 ] ; then


 				#somme md5 (cut=pour récuperer le premier champs avant l'espace)
 				summd5=`md5sum "$PWD/$fichier" | cut -d ' ' -f 1`
 				echo "	RÉFÉRENCE : " "$summd5 : " "$PWD/$fichier" >> $LOG
 			fi

 			#boucle de lecture du tableau
 			for r in "${tabr[@]}"; do

 				#remplace les ";" par de espaces
 				trX=${r//";"/" "}

 				#si une seule ligne = à la référence (ie: le nom du fichier dans le rep courant)
 				if [ "$PWD/$fichier" = "$trX" ] || [ "$nbr_item" = 1 ]; then
 					continue
 				else
 					#il y a des doublons ?
 					sumdoub=`md5sum "$trX" | cut -d ' ' -f 1`

 					#compare la somme MD5 à celle des  autres fichier du même nom
 					if [ "$summd5" = "$sumdoub" ] ; then
 						#c'est un vrai doublon
 						echo "	Doublon   : " "$sumdoub : " "$trX" >> $LOG

 						#le nbr de doublon dans le rep
 						NBRREPDOUB=$((NBRREPDOUB + 1))

 						#le nbr total de doublon dans tous les reps
 						NBRDOUBLON=$((NBRDOUBLON + 1))
 					else
 						#même nom mais pas même somme MD5
 						echo "	Homonyme  : " "$sumdoub : " "$trX" >> $LOG

 					fi
 				fi
 			done
 		fi
	done
	#pluriel
	if [ $NBRREPDOUB -gt 1 ] ; then
		pluriel="doublons"
	else
		pluriel="doublon"
	fi

	echo "------> il y a : " $NBRREPDOUB  " $pluriel"  >> $LOG

}

#explore les repertoires récursivement
function explore_rep
{

local f

	#nombre total de repertoire
	NBRREP=$((NBRREP + 1))

	#sortie console
	echo "$1"

	#pour faire "bo" ;-)
	echo "" >> $LOG

	#mise en forme du fichier de sortie
	if [ $NBRREP = 1 ] || [ $NBRREPDOUB -gt 0 ]; then
		echo "REPERTOIRE : " $1 >> $LOG

	elif [ $NBRREP -gt 1 ] || [ $NBRREPDOUB -gt 0 ]; then
		echo "REPERTOIRE : " $PWD"/"$1 >> $LOG

	fi
	#si pas de rep stop
 	if ! cd "$1" ; then return ; fi

 	#remise à zero du compteur interne de doublon
 	NBRREPDOUB=0

 	#parcour le contenu de répertoire
 	liste_fichier $1

 	#lister les sous-rep
 	for f in * ; do
 		#sauter les liens symboliques
 		if [ -L "$f" ] ; then
 			continue
 		fi

 		#sous rep
 		if [ -d "$f" ] ; then

 			#on recommence avec le sous-rep
 			explore_rep "$f"
 		fi

 	done
 #remonter
 cd ..


 }

#main
#format le fichier de sortie
echo "####### - Le : " $(date +%c) " - ################" >> $LOG
echo "# RACINE : " $1  >> $LOG
echo "#############################################" >> $LOG

explore_rep "$1"



#sortie console
echo "Fin de la recherche "
echo $NBRREP " REPERTOIRE EXPLORÉS"
echo $NBRFICHIER " FICHIERS RECHERCHÉS"
echo $NBRDOUBLON " DOUBLONS TROUVÉS"

#stats fichier
echo "########### - STATS - ################" >> $LOG
echo $NBRREP " REPERTOIRES EXPLORÉS" >> $LOG
echo $NBRFICHIER " FICHIERS RECHERCHÉS" >> $LOG
echo $NBRDOUBLON " DOUBLONS TROUVÉS" >> $LOG
echo "###########################################################" >> $LOG
